/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.emftext.language.java.resource.java.util;

/**
 * This class provides basic infrastructure to interpret models. To implement
 * concrete interpreters, subclass this abstract interpreter and override the
 * interprete_* methods. The interpretation can be customized by binding the two
 * type parameters (ResultType, ContextType). The former is returned by all
 * interprete_* methods, while the latter is passed from method to method while
 * traversing the model. The concrete traversal strategy can also be exchanged.
 * One can use a static traversal strategy by pushing all objects to interpret on
 * the interpretation stack (using addObjectToInterprete()) before calling
 * interprete(). Alternatively, the traversal strategy can be dynamic by pushing
 * objects on the interpretation stack during interpretation.
 */
public class AbstractJavaInterpreter<ResultType, ContextType> {
	
	private java.util.Stack<org.eclipse.emf.ecore.EObject> interpretationStack = new java.util.Stack<org.eclipse.emf.ecore.EObject>();
	private java.util.List<org.emftext.language.java.resource.java.IJavaInterpreterListener> listeners = new java.util.ArrayList<org.emftext.language.java.resource.java.IJavaInterpreterListener>();
	private org.eclipse.emf.ecore.EObject nextObjectToInterprete;
	private Object currentContext;
	
	public ResultType interprete(ContextType context) {
		ResultType result = null;
		org.eclipse.emf.ecore.EObject next = null;
		currentContext = context;
		while (!interpretationStack.empty()) {
			try {
				next = interpretationStack.pop();
			} catch (java.util.EmptyStackException ese) {
				// this can happen when the interpreter was terminated between the call to empty()
				// and pop()
				break;
			}
			nextObjectToInterprete = next;
			notifyListeners(next);
			result = interprete(next, context);
			if (!continueInterpretation(context, result)) {
				break;
			}
		}
		currentContext = null;
		return result;
	}
	
	/**
	 * Override this method to stop the overall interpretation depending on the result
	 * of the interpretation of a single model elements.
	 */
	public boolean continueInterpretation(ContextType context, ResultType result) {
		return true;
	}
	
	public ResultType interprete(org.eclipse.emf.ecore.EObject object, ContextType context) {
		ResultType result = null;
		if (object instanceof org.emftext.language.java.annotations.Annotable) {
			result = interprete_org_emftext_language_java_annotations_Annotable((org.emftext.language.java.annotations.Annotable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.annotations.AnnotationInstance) {
			result = interprete_org_emftext_language_java_annotations_AnnotationInstance((org.emftext.language.java.annotations.AnnotationInstance) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.annotations.SingleAnnotationParameter) {
			result = interprete_org_emftext_language_java_annotations_SingleAnnotationParameter((org.emftext.language.java.annotations.SingleAnnotationParameter) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.annotations.AnnotationParameterList) {
			result = interprete_org_emftext_language_java_annotations_AnnotationParameterList((org.emftext.language.java.annotations.AnnotationParameterList) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.annotations.AnnotationParameter) {
			result = interprete_org_emftext_language_java_annotations_AnnotationParameter((org.emftext.language.java.annotations.AnnotationParameter) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.annotations.AnnotationAttributeSetting) {
			result = interprete_org_emftext_language_java_annotations_AnnotationAttributeSetting((org.emftext.language.java.annotations.AnnotationAttributeSetting) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.arrays.ArrayInstantiationByValuesUntyped) {
			result = interprete_org_emftext_language_java_arrays_ArrayInstantiationByValuesUntyped((org.emftext.language.java.arrays.ArrayInstantiationByValuesUntyped) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.arrays.ArrayInstantiationByValuesTyped) {
			result = interprete_org_emftext_language_java_arrays_ArrayInstantiationByValuesTyped((org.emftext.language.java.arrays.ArrayInstantiationByValuesTyped) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.arrays.ArrayInstantiationByValues) {
			result = interprete_org_emftext_language_java_arrays_ArrayInstantiationByValues((org.emftext.language.java.arrays.ArrayInstantiationByValues) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.MultiplicativeExpression) {
			result = interprete_org_emftext_language_java_expressions_MultiplicativeExpression((org.emftext.language.java.expressions.MultiplicativeExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.UnaryExpression) {
			result = interprete_org_emftext_language_java_expressions_UnaryExpression((org.emftext.language.java.expressions.UnaryExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.PrefixUnaryModificationExpression) {
			result = interprete_org_emftext_language_java_expressions_PrefixUnaryModificationExpression((org.emftext.language.java.expressions.PrefixUnaryModificationExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.SuffixUnaryModificationExpression) {
			result = interprete_org_emftext_language_java_expressions_SuffixUnaryModificationExpression((org.emftext.language.java.expressions.SuffixUnaryModificationExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.UnaryModificationExpression) {
			result = interprete_org_emftext_language_java_expressions_UnaryModificationExpression((org.emftext.language.java.expressions.UnaryModificationExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.CastExpression) {
			result = interprete_org_emftext_language_java_expressions_CastExpression((org.emftext.language.java.expressions.CastExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.NestedExpression) {
			result = interprete_org_emftext_language_java_expressions_NestedExpression((org.emftext.language.java.expressions.NestedExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.instantiations.NewConstructorCall) {
			result = interprete_org_emftext_language_java_instantiations_NewConstructorCall((org.emftext.language.java.instantiations.NewConstructorCall) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.instantiations.ExplicitConstructorCall) {
			result = interprete_org_emftext_language_java_instantiations_ExplicitConstructorCall((org.emftext.language.java.instantiations.ExplicitConstructorCall) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.DecimalFloatLiteral) {
			result = interprete_org_emftext_language_java_literals_DecimalFloatLiteral((org.emftext.language.java.literals.DecimalFloatLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.HexFloatLiteral) {
			result = interprete_org_emftext_language_java_literals_HexFloatLiteral((org.emftext.language.java.literals.HexFloatLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.DecimalDoubleLiteral) {
			result = interprete_org_emftext_language_java_literals_DecimalDoubleLiteral((org.emftext.language.java.literals.DecimalDoubleLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.HexDoubleLiteral) {
			result = interprete_org_emftext_language_java_literals_HexDoubleLiteral((org.emftext.language.java.literals.HexDoubleLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.DoubleLiteral) {
			result = interprete_org_emftext_language_java_literals_DoubleLiteral((org.emftext.language.java.literals.DoubleLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.DecimalIntegerLiteral) {
			result = interprete_org_emftext_language_java_literals_DecimalIntegerLiteral((org.emftext.language.java.literals.DecimalIntegerLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.HexIntegerLiteral) {
			result = interprete_org_emftext_language_java_literals_HexIntegerLiteral((org.emftext.language.java.literals.HexIntegerLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.IntegerLiteral) {
			result = interprete_org_emftext_language_java_literals_IntegerLiteral((org.emftext.language.java.literals.IntegerLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.Literal) {
			result = interprete_org_emftext_language_java_literals_Literal((org.emftext.language.java.literals.Literal) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.PrimaryExpression) {
			result = interprete_org_emftext_language_java_expressions_PrimaryExpression((org.emftext.language.java.expressions.PrimaryExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.UnaryModificationExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_UnaryModificationExpressionChild((org.emftext.language.java.expressions.UnaryModificationExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.UnaryExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_UnaryExpressionChild((org.emftext.language.java.expressions.UnaryExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.MultiplicativeExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_MultiplicativeExpressionChild((org.emftext.language.java.expressions.MultiplicativeExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.annotations.AnnotationValue) {
			result = interprete_org_emftext_language_java_annotations_AnnotationValue((org.emftext.language.java.annotations.AnnotationValue) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.annotations.AnnotationAttribute) {
			result = interprete_org_emftext_language_java_annotations_AnnotationAttribute((org.emftext.language.java.annotations.AnnotationAttribute) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.generics.TypeArgument) {
			result = interprete_org_emftext_language_java_generics_TypeArgument((org.emftext.language.java.generics.TypeArgument) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.members.InterfaceMethod) {
			result = interprete_org_emftext_language_java_members_InterfaceMethod((org.emftext.language.java.members.InterfaceMethod) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.members.ClassMethod) {
			result = interprete_org_emftext_language_java_members_ClassMethod((org.emftext.language.java.members.ClassMethod) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.members.Method) {
			result = interprete_org_emftext_language_java_members_Method((org.emftext.language.java.members.Method) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.arrays.ArrayTypeable) {
			result = interprete_org_emftext_language_java_arrays_ArrayTypeable((org.emftext.language.java.arrays.ArrayTypeable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.arrays.ArrayDimension) {
			result = interprete_org_emftext_language_java_arrays_ArrayDimension((org.emftext.language.java.arrays.ArrayDimension) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.arrays.ArrayInitializer) {
			result = interprete_org_emftext_language_java_arrays_ArrayInitializer((org.emftext.language.java.arrays.ArrayInitializer) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.arrays.ArrayInstantiationBySize) {
			result = interprete_org_emftext_language_java_arrays_ArrayInstantiationBySize((org.emftext.language.java.arrays.ArrayInstantiationBySize) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.arrays.ArrayInstantiation) {
			result = interprete_org_emftext_language_java_arrays_ArrayInstantiation((org.emftext.language.java.arrays.ArrayInstantiation) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.arrays.ArrayInitializationValue) {
			result = interprete_org_emftext_language_java_arrays_ArrayInitializationValue((org.emftext.language.java.arrays.ArrayInitializationValue) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.arrays.ArraySelector) {
			result = interprete_org_emftext_language_java_arrays_ArraySelector((org.emftext.language.java.arrays.ArraySelector) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.classifiers.Class) {
			result = interprete_org_emftext_language_java_classifiers_Class((org.emftext.language.java.classifiers.Class) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.classifiers.Interface) {
			result = interprete_org_emftext_language_java_classifiers_Interface((org.emftext.language.java.classifiers.Interface) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.classifiers.Enumeration) {
			result = interprete_org_emftext_language_java_classifiers_Enumeration((org.emftext.language.java.classifiers.Enumeration) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.classifiers.Annotation) {
			result = interprete_org_emftext_language_java_classifiers_Annotation((org.emftext.language.java.classifiers.Annotation) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.classifiers.ConcreteClassifier) {
			result = interprete_org_emftext_language_java_classifiers_ConcreteClassifier((org.emftext.language.java.classifiers.ConcreteClassifier) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.classifiers.Classifier) {
			result = interprete_org_emftext_language_java_classifiers_Classifier((org.emftext.language.java.classifiers.Classifier) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.classifiers.Implementor) {
			result = interprete_org_emftext_language_java_classifiers_Implementor((org.emftext.language.java.classifiers.Implementor) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.classifiers.AnonymousClass) {
			result = interprete_org_emftext_language_java_classifiers_AnonymousClass((org.emftext.language.java.classifiers.AnonymousClass) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.containers.CompilationUnit) {
			result = interprete_org_emftext_language_java_containers_CompilationUnit((org.emftext.language.java.containers.CompilationUnit) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.containers.Package) {
			result = interprete_org_emftext_language_java_containers_Package((org.emftext.language.java.containers.Package) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.containers.EmptyModel) {
			result = interprete_org_emftext_language_java_containers_EmptyModel((org.emftext.language.java.containers.EmptyModel) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.containers.JavaRoot) {
			result = interprete_org_emftext_language_java_containers_JavaRoot((org.emftext.language.java.containers.JavaRoot) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.members.EnumConstant) {
			result = interprete_org_emftext_language_java_members_EnumConstant((org.emftext.language.java.members.EnumConstant) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.commons.NamedElement) {
			result = interprete_org_emftext_language_java_commons_NamedElement((org.emftext.language.java.commons.NamedElement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.commons.NamespaceAwareElement) {
			result = interprete_org_emftext_language_java_commons_NamespaceAwareElement((org.emftext.language.java.commons.NamespaceAwareElement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.ExpressionList) {
			result = interprete_org_emftext_language_java_expressions_ExpressionList((org.emftext.language.java.expressions.ExpressionList) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.AssignmentExpression) {
			result = interprete_org_emftext_language_java_expressions_AssignmentExpression((org.emftext.language.java.expressions.AssignmentExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.ConditionalExpression) {
			result = interprete_org_emftext_language_java_expressions_ConditionalExpression((org.emftext.language.java.expressions.ConditionalExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.ConditionalOrExpression) {
			result = interprete_org_emftext_language_java_expressions_ConditionalOrExpression((org.emftext.language.java.expressions.ConditionalOrExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.ConditionalAndExpression) {
			result = interprete_org_emftext_language_java_expressions_ConditionalAndExpression((org.emftext.language.java.expressions.ConditionalAndExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.InclusiveOrExpression) {
			result = interprete_org_emftext_language_java_expressions_InclusiveOrExpression((org.emftext.language.java.expressions.InclusiveOrExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.ExclusiveOrExpression) {
			result = interprete_org_emftext_language_java_expressions_ExclusiveOrExpression((org.emftext.language.java.expressions.ExclusiveOrExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.AndExpression) {
			result = interprete_org_emftext_language_java_expressions_AndExpression((org.emftext.language.java.expressions.AndExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.EqualityExpression) {
			result = interprete_org_emftext_language_java_expressions_EqualityExpression((org.emftext.language.java.expressions.EqualityExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.InstanceOfExpression) {
			result = interprete_org_emftext_language_java_expressions_InstanceOfExpression((org.emftext.language.java.expressions.InstanceOfExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.RelationExpression) {
			result = interprete_org_emftext_language_java_expressions_RelationExpression((org.emftext.language.java.expressions.RelationExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.ShiftExpression) {
			result = interprete_org_emftext_language_java_expressions_ShiftExpression((org.emftext.language.java.expressions.ShiftExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.AdditiveExpression) {
			result = interprete_org_emftext_language_java_expressions_AdditiveExpression((org.emftext.language.java.expressions.AdditiveExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.AdditiveExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_AdditiveExpressionChild((org.emftext.language.java.expressions.AdditiveExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.ShiftExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_ShiftExpressionChild((org.emftext.language.java.expressions.ShiftExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.RelationExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_RelationExpressionChild((org.emftext.language.java.expressions.RelationExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.InstanceOfExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_InstanceOfExpressionChild((org.emftext.language.java.expressions.InstanceOfExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.EqualityExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_EqualityExpressionChild((org.emftext.language.java.expressions.EqualityExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.AndExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_AndExpressionChild((org.emftext.language.java.expressions.AndExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.ExclusiveOrExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_ExclusiveOrExpressionChild((org.emftext.language.java.expressions.ExclusiveOrExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.InclusiveOrExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_InclusiveOrExpressionChild((org.emftext.language.java.expressions.InclusiveOrExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.ConditionalAndExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_ConditionalAndExpressionChild((org.emftext.language.java.expressions.ConditionalAndExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.ConditionalOrExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_ConditionalOrExpressionChild((org.emftext.language.java.expressions.ConditionalOrExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.ConditionalExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_ConditionalExpressionChild((org.emftext.language.java.expressions.ConditionalExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.AssignmentExpressionChild) {
			result = interprete_org_emftext_language_java_expressions_AssignmentExpressionChild((org.emftext.language.java.expressions.AssignmentExpressionChild) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.expressions.Expression) {
			result = interprete_org_emftext_language_java_expressions_Expression((org.emftext.language.java.expressions.Expression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.generics.TypeArgumentable) {
			result = interprete_org_emftext_language_java_generics_TypeArgumentable((org.emftext.language.java.generics.TypeArgumentable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.generics.CallTypeArgumentable) {
			result = interprete_org_emftext_language_java_generics_CallTypeArgumentable((org.emftext.language.java.generics.CallTypeArgumentable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.generics.TypeParametrizable) {
			result = interprete_org_emftext_language_java_generics_TypeParametrizable((org.emftext.language.java.generics.TypeParametrizable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.generics.ExtendsTypeArgument) {
			result = interprete_org_emftext_language_java_generics_ExtendsTypeArgument((org.emftext.language.java.generics.ExtendsTypeArgument) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.generics.QualifiedTypeArgument) {
			result = interprete_org_emftext_language_java_generics_QualifiedTypeArgument((org.emftext.language.java.generics.QualifiedTypeArgument) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.generics.SuperTypeArgument) {
			result = interprete_org_emftext_language_java_generics_SuperTypeArgument((org.emftext.language.java.generics.SuperTypeArgument) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.generics.TypeParameter) {
			result = interprete_org_emftext_language_java_generics_TypeParameter((org.emftext.language.java.generics.TypeParameter) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.generics.UnknownTypeArgument) {
			result = interprete_org_emftext_language_java_generics_UnknownTypeArgument((org.emftext.language.java.generics.UnknownTypeArgument) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.imports.Import) {
			result = interprete_org_emftext_language_java_imports_Import((org.emftext.language.java.imports.Import) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.imports.ImportingElement) {
			result = interprete_org_emftext_language_java_imports_ImportingElement((org.emftext.language.java.imports.ImportingElement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.imports.StaticImport) {
			result = interprete_org_emftext_language_java_imports_StaticImport((org.emftext.language.java.imports.StaticImport) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.imports.ClassifierImport) {
			result = interprete_org_emftext_language_java_imports_ClassifierImport((org.emftext.language.java.imports.ClassifierImport) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.imports.PackageImport) {
			result = interprete_org_emftext_language_java_imports_PackageImport((org.emftext.language.java.imports.PackageImport) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.imports.StaticClassifierImport) {
			result = interprete_org_emftext_language_java_imports_StaticClassifierImport((org.emftext.language.java.imports.StaticClassifierImport) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.imports.StaticMemberImport) {
			result = interprete_org_emftext_language_java_imports_StaticMemberImport((org.emftext.language.java.imports.StaticMemberImport) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.instantiations.Initializable) {
			result = interprete_org_emftext_language_java_instantiations_Initializable((org.emftext.language.java.instantiations.Initializable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.instantiations.Instantiation) {
			result = interprete_org_emftext_language_java_instantiations_Instantiation((org.emftext.language.java.instantiations.Instantiation) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.Self) {
			result = interprete_org_emftext_language_java_literals_Self((org.emftext.language.java.literals.Self) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.BooleanLiteral) {
			result = interprete_org_emftext_language_java_literals_BooleanLiteral((org.emftext.language.java.literals.BooleanLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.CharacterLiteral) {
			result = interprete_org_emftext_language_java_literals_CharacterLiteral((org.emftext.language.java.literals.CharacterLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.FloatLiteral) {
			result = interprete_org_emftext_language_java_literals_FloatLiteral((org.emftext.language.java.literals.FloatLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.OctalIntegerLiteral) {
			result = interprete_org_emftext_language_java_literals_OctalIntegerLiteral((org.emftext.language.java.literals.OctalIntegerLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.DecimalLongLiteral) {
			result = interprete_org_emftext_language_java_literals_DecimalLongLiteral((org.emftext.language.java.literals.DecimalLongLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.HexLongLiteral) {
			result = interprete_org_emftext_language_java_literals_HexLongLiteral((org.emftext.language.java.literals.HexLongLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.OctalLongLiteral) {
			result = interprete_org_emftext_language_java_literals_OctalLongLiteral((org.emftext.language.java.literals.OctalLongLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.LongLiteral) {
			result = interprete_org_emftext_language_java_literals_LongLiteral((org.emftext.language.java.literals.LongLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.NullLiteral) {
			result = interprete_org_emftext_language_java_literals_NullLiteral((org.emftext.language.java.literals.NullLiteral) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.Super) {
			result = interprete_org_emftext_language_java_literals_Super((org.emftext.language.java.literals.Super) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.literals.This) {
			result = interprete_org_emftext_language_java_literals_This((org.emftext.language.java.literals.This) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.members.ExceptionThrower) {
			result = interprete_org_emftext_language_java_members_ExceptionThrower((org.emftext.language.java.members.ExceptionThrower) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.members.Member) {
			result = interprete_org_emftext_language_java_members_Member((org.emftext.language.java.members.Member) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.members.MemberContainer) {
			result = interprete_org_emftext_language_java_members_MemberContainer((org.emftext.language.java.members.MemberContainer) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.members.AdditionalField) {
			result = interprete_org_emftext_language_java_members_AdditionalField((org.emftext.language.java.members.AdditionalField) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.members.Constructor) {
			result = interprete_org_emftext_language_java_members_Constructor((org.emftext.language.java.members.Constructor) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.members.EmptyMember) {
			result = interprete_org_emftext_language_java_members_EmptyMember((org.emftext.language.java.members.EmptyMember) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.members.Field) {
			result = interprete_org_emftext_language_java_members_Field((org.emftext.language.java.members.Field) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Modifier) {
			result = interprete_org_emftext_language_java_modifiers_Modifier((org.emftext.language.java.modifiers.Modifier) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.AnnotationInstanceOrModifier) {
			result = interprete_org_emftext_language_java_modifiers_AnnotationInstanceOrModifier((org.emftext.language.java.modifiers.AnnotationInstanceOrModifier) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.AnnotableAndModifiable) {
			result = interprete_org_emftext_language_java_modifiers_AnnotableAndModifiable((org.emftext.language.java.modifiers.AnnotableAndModifiable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Modifiable) {
			result = interprete_org_emftext_language_java_modifiers_Modifiable((org.emftext.language.java.modifiers.Modifiable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Abstract) {
			result = interprete_org_emftext_language_java_modifiers_Abstract((org.emftext.language.java.modifiers.Abstract) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Final) {
			result = interprete_org_emftext_language_java_modifiers_Final((org.emftext.language.java.modifiers.Final) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Native) {
			result = interprete_org_emftext_language_java_modifiers_Native((org.emftext.language.java.modifiers.Native) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Protected) {
			result = interprete_org_emftext_language_java_modifiers_Protected((org.emftext.language.java.modifiers.Protected) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Public) {
			result = interprete_org_emftext_language_java_modifiers_Public((org.emftext.language.java.modifiers.Public) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Private) {
			result = interprete_org_emftext_language_java_modifiers_Private((org.emftext.language.java.modifiers.Private) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Static) {
			result = interprete_org_emftext_language_java_modifiers_Static((org.emftext.language.java.modifiers.Static) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Strictfp) {
			result = interprete_org_emftext_language_java_modifiers_Strictfp((org.emftext.language.java.modifiers.Strictfp) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Synchronized) {
			result = interprete_org_emftext_language_java_modifiers_Synchronized((org.emftext.language.java.modifiers.Synchronized) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Transient) {
			result = interprete_org_emftext_language_java_modifiers_Transient((org.emftext.language.java.modifiers.Transient) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.modifiers.Volatile) {
			result = interprete_org_emftext_language_java_modifiers_Volatile((org.emftext.language.java.modifiers.Volatile) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AdditiveOperator) {
			result = interprete_org_emftext_language_java_operators_AdditiveOperator((org.emftext.language.java.operators.AdditiveOperator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentAnd) {
			result = interprete_org_emftext_language_java_operators_AssignmentAnd((org.emftext.language.java.operators.AssignmentAnd) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentDivision) {
			result = interprete_org_emftext_language_java_operators_AssignmentDivision((org.emftext.language.java.operators.AssignmentDivision) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentExclusiveOr) {
			result = interprete_org_emftext_language_java_operators_AssignmentExclusiveOr((org.emftext.language.java.operators.AssignmentExclusiveOr) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentMinus) {
			result = interprete_org_emftext_language_java_operators_AssignmentMinus((org.emftext.language.java.operators.AssignmentMinus) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentModulo) {
			result = interprete_org_emftext_language_java_operators_AssignmentModulo((org.emftext.language.java.operators.AssignmentModulo) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentMultiplication) {
			result = interprete_org_emftext_language_java_operators_AssignmentMultiplication((org.emftext.language.java.operators.AssignmentMultiplication) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentLeftShift) {
			result = interprete_org_emftext_language_java_operators_AssignmentLeftShift((org.emftext.language.java.operators.AssignmentLeftShift) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentOr) {
			result = interprete_org_emftext_language_java_operators_AssignmentOr((org.emftext.language.java.operators.AssignmentOr) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentPlus) {
			result = interprete_org_emftext_language_java_operators_AssignmentPlus((org.emftext.language.java.operators.AssignmentPlus) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentRightShift) {
			result = interprete_org_emftext_language_java_operators_AssignmentRightShift((org.emftext.language.java.operators.AssignmentRightShift) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentUnsignedRightShift) {
			result = interprete_org_emftext_language_java_operators_AssignmentUnsignedRightShift((org.emftext.language.java.operators.AssignmentUnsignedRightShift) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.AssignmentOperator) {
			result = interprete_org_emftext_language_java_operators_AssignmentOperator((org.emftext.language.java.operators.AssignmentOperator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.Equal) {
			result = interprete_org_emftext_language_java_operators_Equal((org.emftext.language.java.operators.Equal) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.NotEqual) {
			result = interprete_org_emftext_language_java_operators_NotEqual((org.emftext.language.java.operators.NotEqual) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.EqualityOperator) {
			result = interprete_org_emftext_language_java_operators_EqualityOperator((org.emftext.language.java.operators.EqualityOperator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.MultiplicativeOperator) {
			result = interprete_org_emftext_language_java_operators_MultiplicativeOperator((org.emftext.language.java.operators.MultiplicativeOperator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.GreaterThan) {
			result = interprete_org_emftext_language_java_operators_GreaterThan((org.emftext.language.java.operators.GreaterThan) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.GreaterThanOrEqual) {
			result = interprete_org_emftext_language_java_operators_GreaterThanOrEqual((org.emftext.language.java.operators.GreaterThanOrEqual) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.LessThan) {
			result = interprete_org_emftext_language_java_operators_LessThan((org.emftext.language.java.operators.LessThan) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.LessThanOrEqual) {
			result = interprete_org_emftext_language_java_operators_LessThanOrEqual((org.emftext.language.java.operators.LessThanOrEqual) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.RelationOperator) {
			result = interprete_org_emftext_language_java_operators_RelationOperator((org.emftext.language.java.operators.RelationOperator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.ShiftOperator) {
			result = interprete_org_emftext_language_java_operators_ShiftOperator((org.emftext.language.java.operators.ShiftOperator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.Addition) {
			result = interprete_org_emftext_language_java_operators_Addition((org.emftext.language.java.operators.Addition) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.Subtraction) {
			result = interprete_org_emftext_language_java_operators_Subtraction((org.emftext.language.java.operators.Subtraction) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.UnaryOperator) {
			result = interprete_org_emftext_language_java_operators_UnaryOperator((org.emftext.language.java.operators.UnaryOperator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.UnaryModificationOperator) {
			result = interprete_org_emftext_language_java_operators_UnaryModificationOperator((org.emftext.language.java.operators.UnaryModificationOperator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.Assignment) {
			result = interprete_org_emftext_language_java_operators_Assignment((org.emftext.language.java.operators.Assignment) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.Division) {
			result = interprete_org_emftext_language_java_operators_Division((org.emftext.language.java.operators.Division) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.Multiplication) {
			result = interprete_org_emftext_language_java_operators_Multiplication((org.emftext.language.java.operators.Multiplication) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.Remainder) {
			result = interprete_org_emftext_language_java_operators_Remainder((org.emftext.language.java.operators.Remainder) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.Complement) {
			result = interprete_org_emftext_language_java_operators_Complement((org.emftext.language.java.operators.Complement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.MinusMinus) {
			result = interprete_org_emftext_language_java_operators_MinusMinus((org.emftext.language.java.operators.MinusMinus) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.Negate) {
			result = interprete_org_emftext_language_java_operators_Negate((org.emftext.language.java.operators.Negate) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.PlusPlus) {
			result = interprete_org_emftext_language_java_operators_PlusPlus((org.emftext.language.java.operators.PlusPlus) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.LeftShift) {
			result = interprete_org_emftext_language_java_operators_LeftShift((org.emftext.language.java.operators.LeftShift) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.RightShift) {
			result = interprete_org_emftext_language_java_operators_RightShift((org.emftext.language.java.operators.RightShift) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.UnsignedRightShift) {
			result = interprete_org_emftext_language_java_operators_UnsignedRightShift((org.emftext.language.java.operators.UnsignedRightShift) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.operators.Operator) {
			result = interprete_org_emftext_language_java_operators_Operator((org.emftext.language.java.operators.Operator) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.parameters.Parameter) {
			result = interprete_org_emftext_language_java_parameters_Parameter((org.emftext.language.java.parameters.Parameter) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.parameters.Parametrizable) {
			result = interprete_org_emftext_language_java_parameters_Parametrizable((org.emftext.language.java.parameters.Parametrizable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.parameters.OrdinaryParameter) {
			result = interprete_org_emftext_language_java_parameters_OrdinaryParameter((org.emftext.language.java.parameters.OrdinaryParameter) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.parameters.VariableLengthParameter) {
			result = interprete_org_emftext_language_java_parameters_VariableLengthParameter((org.emftext.language.java.parameters.VariableLengthParameter) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.references.IdentifierReference) {
			result = interprete_org_emftext_language_java_references_IdentifierReference((org.emftext.language.java.references.IdentifierReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.references.MethodCall) {
			result = interprete_org_emftext_language_java_references_MethodCall((org.emftext.language.java.references.MethodCall) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.references.ReflectiveClassReference) {
			result = interprete_org_emftext_language_java_references_ReflectiveClassReference((org.emftext.language.java.references.ReflectiveClassReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.references.PrimitiveTypeReference) {
			result = interprete_org_emftext_language_java_references_PrimitiveTypeReference((org.emftext.language.java.references.PrimitiveTypeReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.references.StringReference) {
			result = interprete_org_emftext_language_java_references_StringReference((org.emftext.language.java.references.StringReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.references.SelfReference) {
			result = interprete_org_emftext_language_java_references_SelfReference((org.emftext.language.java.references.SelfReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.references.Reference) {
			result = interprete_org_emftext_language_java_references_Reference((org.emftext.language.java.references.Reference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.references.Argumentable) {
			result = interprete_org_emftext_language_java_references_Argumentable((org.emftext.language.java.references.Argumentable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.references.PackageReference) {
			result = interprete_org_emftext_language_java_references_PackageReference((org.emftext.language.java.references.PackageReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.references.ReferenceableElement) {
			result = interprete_org_emftext_language_java_references_ReferenceableElement((org.emftext.language.java.references.ReferenceableElement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.references.ElementReference) {
			result = interprete_org_emftext_language_java_references_ElementReference((org.emftext.language.java.references.ElementReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.StatementContainer) {
			result = interprete_org_emftext_language_java_statements_StatementContainer((org.emftext.language.java.statements.StatementContainer) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.StatementListContainer) {
			result = interprete_org_emftext_language_java_statements_StatementListContainer((org.emftext.language.java.statements.StatementListContainer) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.Conditional) {
			result = interprete_org_emftext_language_java_statements_Conditional((org.emftext.language.java.statements.Conditional) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.ForLoopInitializer) {
			result = interprete_org_emftext_language_java_statements_ForLoopInitializer((org.emftext.language.java.statements.ForLoopInitializer) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.Statement) {
			result = interprete_org_emftext_language_java_statements_Statement((org.emftext.language.java.statements.Statement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.SwitchCase) {
			result = interprete_org_emftext_language_java_statements_SwitchCase((org.emftext.language.java.statements.SwitchCase) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.Assert) {
			result = interprete_org_emftext_language_java_statements_Assert((org.emftext.language.java.statements.Assert) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.Break) {
			result = interprete_org_emftext_language_java_statements_Break((org.emftext.language.java.statements.Break) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.Block) {
			result = interprete_org_emftext_language_java_statements_Block((org.emftext.language.java.statements.Block) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.CatchBlock) {
			result = interprete_org_emftext_language_java_statements_CatchBlock((org.emftext.language.java.statements.CatchBlock) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.Condition) {
			result = interprete_org_emftext_language_java_statements_Condition((org.emftext.language.java.statements.Condition) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.Continue) {
			result = interprete_org_emftext_language_java_statements_Continue((org.emftext.language.java.statements.Continue) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.DefaultSwitchCase) {
			result = interprete_org_emftext_language_java_statements_DefaultSwitchCase((org.emftext.language.java.statements.DefaultSwitchCase) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.DoWhileLoop) {
			result = interprete_org_emftext_language_java_statements_DoWhileLoop((org.emftext.language.java.statements.DoWhileLoop) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.EmptyStatement) {
			result = interprete_org_emftext_language_java_statements_EmptyStatement((org.emftext.language.java.statements.EmptyStatement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.ExpressionStatement) {
			result = interprete_org_emftext_language_java_statements_ExpressionStatement((org.emftext.language.java.statements.ExpressionStatement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.ForLoop) {
			result = interprete_org_emftext_language_java_statements_ForLoop((org.emftext.language.java.statements.ForLoop) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.ForEachLoop) {
			result = interprete_org_emftext_language_java_statements_ForEachLoop((org.emftext.language.java.statements.ForEachLoop) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.Jump) {
			result = interprete_org_emftext_language_java_statements_Jump((org.emftext.language.java.statements.Jump) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.JumpLabel) {
			result = interprete_org_emftext_language_java_statements_JumpLabel((org.emftext.language.java.statements.JumpLabel) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.LocalVariableStatement) {
			result = interprete_org_emftext_language_java_statements_LocalVariableStatement((org.emftext.language.java.statements.LocalVariableStatement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.NormalSwitchCase) {
			result = interprete_org_emftext_language_java_statements_NormalSwitchCase((org.emftext.language.java.statements.NormalSwitchCase) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.Return) {
			result = interprete_org_emftext_language_java_statements_Return((org.emftext.language.java.statements.Return) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.Switch) {
			result = interprete_org_emftext_language_java_statements_Switch((org.emftext.language.java.statements.Switch) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.SynchronizedBlock) {
			result = interprete_org_emftext_language_java_statements_SynchronizedBlock((org.emftext.language.java.statements.SynchronizedBlock) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.Throw) {
			result = interprete_org_emftext_language_java_statements_Throw((org.emftext.language.java.statements.Throw) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.TryBlock) {
			result = interprete_org_emftext_language_java_statements_TryBlock((org.emftext.language.java.statements.TryBlock) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.statements.WhileLoop) {
			result = interprete_org_emftext_language_java_statements_WhileLoop((org.emftext.language.java.statements.WhileLoop) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.Type) {
			result = interprete_org_emftext_language_java_types_Type((org.emftext.language.java.types.Type) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.TypedElement) {
			result = interprete_org_emftext_language_java_types_TypedElement((org.emftext.language.java.types.TypedElement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.ClassifierReference) {
			result = interprete_org_emftext_language_java_types_ClassifierReference((org.emftext.language.java.types.ClassifierReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.NamespaceClassifierReference) {
			result = interprete_org_emftext_language_java_types_NamespaceClassifierReference((org.emftext.language.java.types.NamespaceClassifierReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.Boolean) {
			result = interprete_org_emftext_language_java_types_Boolean((org.emftext.language.java.types.Boolean) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.Byte) {
			result = interprete_org_emftext_language_java_types_Byte((org.emftext.language.java.types.Byte) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.Char) {
			result = interprete_org_emftext_language_java_types_Char((org.emftext.language.java.types.Char) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.Double) {
			result = interprete_org_emftext_language_java_types_Double((org.emftext.language.java.types.Double) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.Float) {
			result = interprete_org_emftext_language_java_types_Float((org.emftext.language.java.types.Float) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.Int) {
			result = interprete_org_emftext_language_java_types_Int((org.emftext.language.java.types.Int) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.Long) {
			result = interprete_org_emftext_language_java_types_Long((org.emftext.language.java.types.Long) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.Short) {
			result = interprete_org_emftext_language_java_types_Short((org.emftext.language.java.types.Short) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.Void) {
			result = interprete_org_emftext_language_java_types_Void((org.emftext.language.java.types.Void) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.PrimitiveType) {
			result = interprete_org_emftext_language_java_types_PrimitiveType((org.emftext.language.java.types.PrimitiveType) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.types.TypeReference) {
			result = interprete_org_emftext_language_java_types_TypeReference((org.emftext.language.java.types.TypeReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.variables.LocalVariable) {
			result = interprete_org_emftext_language_java_variables_LocalVariable((org.emftext.language.java.variables.LocalVariable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.variables.Variable) {
			result = interprete_org_emftext_language_java_variables_Variable((org.emftext.language.java.variables.Variable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.variables.AdditionalLocalVariable) {
			result = interprete_org_emftext_language_java_variables_AdditionalLocalVariable((org.emftext.language.java.variables.AdditionalLocalVariable) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.java.commons.Commentable) {
			result = interprete_org_emftext_language_java_commons_Commentable((org.emftext.language.java.commons.Commentable) object, context);
		}
		if (result != null) {
			return result;
		}
		return result;
	}
	
	public ResultType interprete_org_emftext_language_java_annotations_Annotable(org.emftext.language.java.annotations.Annotable annotable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_annotations_AnnotationInstance(org.emftext.language.java.annotations.AnnotationInstance annotationInstance, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_annotations_AnnotationParameter(org.emftext.language.java.annotations.AnnotationParameter annotationParameter, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_annotations_SingleAnnotationParameter(org.emftext.language.java.annotations.SingleAnnotationParameter singleAnnotationParameter, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_annotations_AnnotationParameterList(org.emftext.language.java.annotations.AnnotationParameterList annotationParameterList, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_annotations_AnnotationAttributeSetting(org.emftext.language.java.annotations.AnnotationAttributeSetting annotationAttributeSetting, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_annotations_AnnotationValue(org.emftext.language.java.annotations.AnnotationValue annotationValue, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_annotations_AnnotationAttribute(org.emftext.language.java.annotations.AnnotationAttribute annotationAttribute, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_arrays_ArrayTypeable(org.emftext.language.java.arrays.ArrayTypeable arrayTypeable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_arrays_ArrayDimension(org.emftext.language.java.arrays.ArrayDimension arrayDimension, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_arrays_ArrayInitializer(org.emftext.language.java.arrays.ArrayInitializer arrayInitializer, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_arrays_ArrayInitializationValue(org.emftext.language.java.arrays.ArrayInitializationValue arrayInitializationValue, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_arrays_ArrayInstantiation(org.emftext.language.java.arrays.ArrayInstantiation arrayInstantiation, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_arrays_ArrayInstantiationBySize(org.emftext.language.java.arrays.ArrayInstantiationBySize arrayInstantiationBySize, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_arrays_ArrayInstantiationByValues(org.emftext.language.java.arrays.ArrayInstantiationByValues arrayInstantiationByValues, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_arrays_ArrayInstantiationByValuesUntyped(org.emftext.language.java.arrays.ArrayInstantiationByValuesUntyped arrayInstantiationByValuesUntyped, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_arrays_ArrayInstantiationByValuesTyped(org.emftext.language.java.arrays.ArrayInstantiationByValuesTyped arrayInstantiationByValuesTyped, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_arrays_ArraySelector(org.emftext.language.java.arrays.ArraySelector arraySelector, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_classifiers_Classifier(org.emftext.language.java.classifiers.Classifier classifier, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_classifiers_ConcreteClassifier(org.emftext.language.java.classifiers.ConcreteClassifier concreteClassifier, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_classifiers_Implementor(org.emftext.language.java.classifiers.Implementor implementor, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_classifiers_Class(org.emftext.language.java.classifiers.Class _class, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_classifiers_Interface(org.emftext.language.java.classifiers.Interface _interface, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_classifiers_Enumeration(org.emftext.language.java.classifiers.Enumeration enumeration, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_classifiers_Annotation(org.emftext.language.java.classifiers.Annotation annotation, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_classifiers_AnonymousClass(org.emftext.language.java.classifiers.AnonymousClass anonymousClass, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_commons_Commentable(org.emftext.language.java.commons.Commentable commentable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_commons_NamedElement(org.emftext.language.java.commons.NamedElement namedElement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_commons_NamespaceAwareElement(org.emftext.language.java.commons.NamespaceAwareElement namespaceAwareElement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_containers_JavaRoot(org.emftext.language.java.containers.JavaRoot javaRoot, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_containers_CompilationUnit(org.emftext.language.java.containers.CompilationUnit compilationUnit, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_containers_Package(org.emftext.language.java.containers.Package _package, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_containers_EmptyModel(org.emftext.language.java.containers.EmptyModel emptyModel, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_ExpressionList(org.emftext.language.java.expressions.ExpressionList expressionList, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_Expression(org.emftext.language.java.expressions.Expression expression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_AssignmentExpression(org.emftext.language.java.expressions.AssignmentExpression assignmentExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_AssignmentExpressionChild(org.emftext.language.java.expressions.AssignmentExpressionChild assignmentExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_ConditionalExpression(org.emftext.language.java.expressions.ConditionalExpression conditionalExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_ConditionalExpressionChild(org.emftext.language.java.expressions.ConditionalExpressionChild conditionalExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_ConditionalOrExpression(org.emftext.language.java.expressions.ConditionalOrExpression conditionalOrExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_ConditionalOrExpressionChild(org.emftext.language.java.expressions.ConditionalOrExpressionChild conditionalOrExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_ConditionalAndExpression(org.emftext.language.java.expressions.ConditionalAndExpression conditionalAndExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_ConditionalAndExpressionChild(org.emftext.language.java.expressions.ConditionalAndExpressionChild conditionalAndExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_InclusiveOrExpression(org.emftext.language.java.expressions.InclusiveOrExpression inclusiveOrExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_InclusiveOrExpressionChild(org.emftext.language.java.expressions.InclusiveOrExpressionChild inclusiveOrExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_ExclusiveOrExpression(org.emftext.language.java.expressions.ExclusiveOrExpression exclusiveOrExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_ExclusiveOrExpressionChild(org.emftext.language.java.expressions.ExclusiveOrExpressionChild exclusiveOrExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_AndExpression(org.emftext.language.java.expressions.AndExpression andExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_AndExpressionChild(org.emftext.language.java.expressions.AndExpressionChild andExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_EqualityExpression(org.emftext.language.java.expressions.EqualityExpression equalityExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_EqualityExpressionChild(org.emftext.language.java.expressions.EqualityExpressionChild equalityExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_InstanceOfExpression(org.emftext.language.java.expressions.InstanceOfExpression instanceOfExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_InstanceOfExpressionChild(org.emftext.language.java.expressions.InstanceOfExpressionChild instanceOfExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_RelationExpression(org.emftext.language.java.expressions.RelationExpression relationExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_RelationExpressionChild(org.emftext.language.java.expressions.RelationExpressionChild relationExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_ShiftExpression(org.emftext.language.java.expressions.ShiftExpression shiftExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_ShiftExpressionChild(org.emftext.language.java.expressions.ShiftExpressionChild shiftExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_AdditiveExpression(org.emftext.language.java.expressions.AdditiveExpression additiveExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_AdditiveExpressionChild(org.emftext.language.java.expressions.AdditiveExpressionChild additiveExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_MultiplicativeExpression(org.emftext.language.java.expressions.MultiplicativeExpression multiplicativeExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_MultiplicativeExpressionChild(org.emftext.language.java.expressions.MultiplicativeExpressionChild multiplicativeExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_UnaryExpression(org.emftext.language.java.expressions.UnaryExpression unaryExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_UnaryExpressionChild(org.emftext.language.java.expressions.UnaryExpressionChild unaryExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_UnaryModificationExpression(org.emftext.language.java.expressions.UnaryModificationExpression unaryModificationExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_PrefixUnaryModificationExpression(org.emftext.language.java.expressions.PrefixUnaryModificationExpression prefixUnaryModificationExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_SuffixUnaryModificationExpression(org.emftext.language.java.expressions.SuffixUnaryModificationExpression suffixUnaryModificationExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_UnaryModificationExpressionChild(org.emftext.language.java.expressions.UnaryModificationExpressionChild unaryModificationExpressionChild, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_CastExpression(org.emftext.language.java.expressions.CastExpression castExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_PrimaryExpression(org.emftext.language.java.expressions.PrimaryExpression primaryExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_expressions_NestedExpression(org.emftext.language.java.expressions.NestedExpression nestedExpression, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_generics_TypeArgument(org.emftext.language.java.generics.TypeArgument typeArgument, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_generics_TypeArgumentable(org.emftext.language.java.generics.TypeArgumentable typeArgumentable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_generics_CallTypeArgumentable(org.emftext.language.java.generics.CallTypeArgumentable callTypeArgumentable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_generics_TypeParametrizable(org.emftext.language.java.generics.TypeParametrizable typeParametrizable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_generics_ExtendsTypeArgument(org.emftext.language.java.generics.ExtendsTypeArgument extendsTypeArgument, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_generics_QualifiedTypeArgument(org.emftext.language.java.generics.QualifiedTypeArgument qualifiedTypeArgument, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_generics_SuperTypeArgument(org.emftext.language.java.generics.SuperTypeArgument superTypeArgument, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_generics_TypeParameter(org.emftext.language.java.generics.TypeParameter typeParameter, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_generics_UnknownTypeArgument(org.emftext.language.java.generics.UnknownTypeArgument unknownTypeArgument, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_imports_Import(org.emftext.language.java.imports.Import _import, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_imports_ImportingElement(org.emftext.language.java.imports.ImportingElement importingElement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_imports_StaticImport(org.emftext.language.java.imports.StaticImport staticImport, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_imports_ClassifierImport(org.emftext.language.java.imports.ClassifierImport classifierImport, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_imports_PackageImport(org.emftext.language.java.imports.PackageImport packageImport, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_imports_StaticClassifierImport(org.emftext.language.java.imports.StaticClassifierImport staticClassifierImport, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_imports_StaticMemberImport(org.emftext.language.java.imports.StaticMemberImport staticMemberImport, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_instantiations_Initializable(org.emftext.language.java.instantiations.Initializable initializable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_instantiations_Instantiation(org.emftext.language.java.instantiations.Instantiation instantiation, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_instantiations_NewConstructorCall(org.emftext.language.java.instantiations.NewConstructorCall newConstructorCall, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_instantiations_ExplicitConstructorCall(org.emftext.language.java.instantiations.ExplicitConstructorCall explicitConstructorCall, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_Literal(org.emftext.language.java.literals.Literal literal, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_Self(org.emftext.language.java.literals.Self self, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_BooleanLiteral(org.emftext.language.java.literals.BooleanLiteral booleanLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_CharacterLiteral(org.emftext.language.java.literals.CharacterLiteral characterLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_FloatLiteral(org.emftext.language.java.literals.FloatLiteral floatLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_DecimalFloatLiteral(org.emftext.language.java.literals.DecimalFloatLiteral decimalFloatLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_HexFloatLiteral(org.emftext.language.java.literals.HexFloatLiteral hexFloatLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_DoubleLiteral(org.emftext.language.java.literals.DoubleLiteral doubleLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_DecimalDoubleLiteral(org.emftext.language.java.literals.DecimalDoubleLiteral decimalDoubleLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_HexDoubleLiteral(org.emftext.language.java.literals.HexDoubleLiteral hexDoubleLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_IntegerLiteral(org.emftext.language.java.literals.IntegerLiteral integerLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_DecimalIntegerLiteral(org.emftext.language.java.literals.DecimalIntegerLiteral decimalIntegerLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_HexIntegerLiteral(org.emftext.language.java.literals.HexIntegerLiteral hexIntegerLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_OctalIntegerLiteral(org.emftext.language.java.literals.OctalIntegerLiteral octalIntegerLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_LongLiteral(org.emftext.language.java.literals.LongLiteral longLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_DecimalLongLiteral(org.emftext.language.java.literals.DecimalLongLiteral decimalLongLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_HexLongLiteral(org.emftext.language.java.literals.HexLongLiteral hexLongLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_OctalLongLiteral(org.emftext.language.java.literals.OctalLongLiteral octalLongLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_NullLiteral(org.emftext.language.java.literals.NullLiteral nullLiteral, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_Super(org.emftext.language.java.literals.Super _super, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_literals_This(org.emftext.language.java.literals.This _this, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_members_ExceptionThrower(org.emftext.language.java.members.ExceptionThrower exceptionThrower, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_members_Member(org.emftext.language.java.members.Member member, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_members_MemberContainer(org.emftext.language.java.members.MemberContainer memberContainer, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_members_AdditionalField(org.emftext.language.java.members.AdditionalField additionalField, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_members_Constructor(org.emftext.language.java.members.Constructor constructor, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_members_EmptyMember(org.emftext.language.java.members.EmptyMember emptyMember, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_members_Field(org.emftext.language.java.members.Field field, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_members_Method(org.emftext.language.java.members.Method method, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_members_InterfaceMethod(org.emftext.language.java.members.InterfaceMethod interfaceMethod, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_members_ClassMethod(org.emftext.language.java.members.ClassMethod classMethod, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_members_EnumConstant(org.emftext.language.java.members.EnumConstant enumConstant, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Modifier(org.emftext.language.java.modifiers.Modifier modifier, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_AnnotationInstanceOrModifier(org.emftext.language.java.modifiers.AnnotationInstanceOrModifier annotationInstanceOrModifier, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_AnnotableAndModifiable(org.emftext.language.java.modifiers.AnnotableAndModifiable annotableAndModifiable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Modifiable(org.emftext.language.java.modifiers.Modifiable modifiable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Abstract(org.emftext.language.java.modifiers.Abstract _abstract, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Final(org.emftext.language.java.modifiers.Final _final, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Native(org.emftext.language.java.modifiers.Native _native, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Protected(org.emftext.language.java.modifiers.Protected _protected, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Public(org.emftext.language.java.modifiers.Public _public, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Private(org.emftext.language.java.modifiers.Private _private, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Static(org.emftext.language.java.modifiers.Static _static, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Strictfp(org.emftext.language.java.modifiers.Strictfp _strictfp, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Synchronized(org.emftext.language.java.modifiers.Synchronized _synchronized, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Transient(org.emftext.language.java.modifiers.Transient _transient, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_modifiers_Volatile(org.emftext.language.java.modifiers.Volatile _volatile, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_Operator(org.emftext.language.java.operators.Operator operator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AdditiveOperator(org.emftext.language.java.operators.AdditiveOperator additiveOperator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentOperator(org.emftext.language.java.operators.AssignmentOperator assignmentOperator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_EqualityOperator(org.emftext.language.java.operators.EqualityOperator equalityOperator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_MultiplicativeOperator(org.emftext.language.java.operators.MultiplicativeOperator multiplicativeOperator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_RelationOperator(org.emftext.language.java.operators.RelationOperator relationOperator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_ShiftOperator(org.emftext.language.java.operators.ShiftOperator shiftOperator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_UnaryOperator(org.emftext.language.java.operators.UnaryOperator unaryOperator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_UnaryModificationOperator(org.emftext.language.java.operators.UnaryModificationOperator unaryModificationOperator, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_Assignment(org.emftext.language.java.operators.Assignment assignment, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentAnd(org.emftext.language.java.operators.AssignmentAnd assignmentAnd, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentDivision(org.emftext.language.java.operators.AssignmentDivision assignmentDivision, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentExclusiveOr(org.emftext.language.java.operators.AssignmentExclusiveOr assignmentExclusiveOr, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentMinus(org.emftext.language.java.operators.AssignmentMinus assignmentMinus, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentModulo(org.emftext.language.java.operators.AssignmentModulo assignmentModulo, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentMultiplication(org.emftext.language.java.operators.AssignmentMultiplication assignmentMultiplication, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentLeftShift(org.emftext.language.java.operators.AssignmentLeftShift assignmentLeftShift, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentOr(org.emftext.language.java.operators.AssignmentOr assignmentOr, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentPlus(org.emftext.language.java.operators.AssignmentPlus assignmentPlus, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentRightShift(org.emftext.language.java.operators.AssignmentRightShift assignmentRightShift, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_AssignmentUnsignedRightShift(org.emftext.language.java.operators.AssignmentUnsignedRightShift assignmentUnsignedRightShift, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_Equal(org.emftext.language.java.operators.Equal equal, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_NotEqual(org.emftext.language.java.operators.NotEqual notEqual, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_GreaterThan(org.emftext.language.java.operators.GreaterThan greaterThan, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_GreaterThanOrEqual(org.emftext.language.java.operators.GreaterThanOrEqual greaterThanOrEqual, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_LessThan(org.emftext.language.java.operators.LessThan lessThan, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_LessThanOrEqual(org.emftext.language.java.operators.LessThanOrEqual lessThanOrEqual, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_Addition(org.emftext.language.java.operators.Addition addition, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_Subtraction(org.emftext.language.java.operators.Subtraction subtraction, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_Division(org.emftext.language.java.operators.Division division, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_Multiplication(org.emftext.language.java.operators.Multiplication multiplication, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_Remainder(org.emftext.language.java.operators.Remainder remainder, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_Complement(org.emftext.language.java.operators.Complement complement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_MinusMinus(org.emftext.language.java.operators.MinusMinus minusMinus, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_Negate(org.emftext.language.java.operators.Negate negate, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_PlusPlus(org.emftext.language.java.operators.PlusPlus plusPlus, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_LeftShift(org.emftext.language.java.operators.LeftShift leftShift, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_RightShift(org.emftext.language.java.operators.RightShift rightShift, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_operators_UnsignedRightShift(org.emftext.language.java.operators.UnsignedRightShift unsignedRightShift, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_parameters_Parameter(org.emftext.language.java.parameters.Parameter parameter, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_parameters_Parametrizable(org.emftext.language.java.parameters.Parametrizable parametrizable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_parameters_OrdinaryParameter(org.emftext.language.java.parameters.OrdinaryParameter ordinaryParameter, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_parameters_VariableLengthParameter(org.emftext.language.java.parameters.VariableLengthParameter variableLengthParameter, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_references_Reference(org.emftext.language.java.references.Reference reference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_references_Argumentable(org.emftext.language.java.references.Argumentable argumentable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_references_ReferenceableElement(org.emftext.language.java.references.ReferenceableElement referenceableElement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_references_ElementReference(org.emftext.language.java.references.ElementReference elementReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_references_IdentifierReference(org.emftext.language.java.references.IdentifierReference identifierReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_references_MethodCall(org.emftext.language.java.references.MethodCall methodCall, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_references_ReflectiveClassReference(org.emftext.language.java.references.ReflectiveClassReference reflectiveClassReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_references_PrimitiveTypeReference(org.emftext.language.java.references.PrimitiveTypeReference primitiveTypeReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_references_StringReference(org.emftext.language.java.references.StringReference stringReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_references_SelfReference(org.emftext.language.java.references.SelfReference selfReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_references_PackageReference(org.emftext.language.java.references.PackageReference packageReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_StatementContainer(org.emftext.language.java.statements.StatementContainer statementContainer, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_StatementListContainer(org.emftext.language.java.statements.StatementListContainer statementListContainer, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_Conditional(org.emftext.language.java.statements.Conditional conditional, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_ForLoopInitializer(org.emftext.language.java.statements.ForLoopInitializer forLoopInitializer, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_Statement(org.emftext.language.java.statements.Statement statement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_SwitchCase(org.emftext.language.java.statements.SwitchCase switchCase, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_Assert(org.emftext.language.java.statements.Assert _assert, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_Break(org.emftext.language.java.statements.Break _break, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_Block(org.emftext.language.java.statements.Block block, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_CatchBlock(org.emftext.language.java.statements.CatchBlock catchBlock, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_Condition(org.emftext.language.java.statements.Condition condition, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_Continue(org.emftext.language.java.statements.Continue _continue, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_DefaultSwitchCase(org.emftext.language.java.statements.DefaultSwitchCase defaultSwitchCase, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_DoWhileLoop(org.emftext.language.java.statements.DoWhileLoop doWhileLoop, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_EmptyStatement(org.emftext.language.java.statements.EmptyStatement emptyStatement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_ExpressionStatement(org.emftext.language.java.statements.ExpressionStatement expressionStatement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_ForLoop(org.emftext.language.java.statements.ForLoop forLoop, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_ForEachLoop(org.emftext.language.java.statements.ForEachLoop forEachLoop, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_Jump(org.emftext.language.java.statements.Jump jump, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_JumpLabel(org.emftext.language.java.statements.JumpLabel jumpLabel, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_LocalVariableStatement(org.emftext.language.java.statements.LocalVariableStatement localVariableStatement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_NormalSwitchCase(org.emftext.language.java.statements.NormalSwitchCase normalSwitchCase, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_Return(org.emftext.language.java.statements.Return _return, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_Switch(org.emftext.language.java.statements.Switch _switch, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_SynchronizedBlock(org.emftext.language.java.statements.SynchronizedBlock synchronizedBlock, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_Throw(org.emftext.language.java.statements.Throw _throw, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_TryBlock(org.emftext.language.java.statements.TryBlock tryBlock, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_statements_WhileLoop(org.emftext.language.java.statements.WhileLoop whileLoop, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_Type(org.emftext.language.java.types.Type type, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_TypedElement(org.emftext.language.java.types.TypedElement typedElement, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_TypeReference(org.emftext.language.java.types.TypeReference typeReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_ClassifierReference(org.emftext.language.java.types.ClassifierReference classifierReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_NamespaceClassifierReference(org.emftext.language.java.types.NamespaceClassifierReference namespaceClassifierReference, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_PrimitiveType(org.emftext.language.java.types.PrimitiveType primitiveType, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_Boolean(org.emftext.language.java.types.Boolean _boolean, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_Byte(org.emftext.language.java.types.Byte _byte, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_Char(org.emftext.language.java.types.Char _char, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_Double(org.emftext.language.java.types.Double _double, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_Float(org.emftext.language.java.types.Float _float, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_Int(org.emftext.language.java.types.Int _int, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_Long(org.emftext.language.java.types.Long _long, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_Short(org.emftext.language.java.types.Short _short, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_types_Void(org.emftext.language.java.types.Void _void, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_variables_Variable(org.emftext.language.java.variables.Variable variable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_variables_LocalVariable(org.emftext.language.java.variables.LocalVariable localVariable, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_java_variables_AdditionalLocalVariable(org.emftext.language.java.variables.AdditionalLocalVariable additionalLocalVariable, ContextType context) {
		return null;
	}
	
	private void notifyListeners(org.eclipse.emf.ecore.EObject element) {
		for (org.emftext.language.java.resource.java.IJavaInterpreterListener listener : listeners) {
			listener.handleInterpreteObject(element);
		}
	}
	
	/**
	 * Adds the given object to the interpretation stack. Attention: Objects that are
	 * added first, are interpret last.
	 */
	public void addObjectToInterprete(org.eclipse.emf.ecore.EObject object) {
		interpretationStack.push(object);
	}
	
	/**
	 * Adds the given collection of objects to the interpretation stack. Attention:
	 * Collections that are added first, are interpret last.
	 */
	public void addObjectsToInterprete(java.util.Collection<? extends org.eclipse.emf.ecore.EObject> objects) {
		for (org.eclipse.emf.ecore.EObject object : objects) {
			addObjectToInterprete(object);
		}
	}
	
	/**
	 * Adds the given collection of objects in reverse order to the interpretation
	 * stack.
	 */
	public void addObjectsToInterpreteInReverseOrder(java.util.Collection<? extends org.eclipse.emf.ecore.EObject> objects) {
		java.util.List<org.eclipse.emf.ecore.EObject> reverse = new java.util.ArrayList<org.eclipse.emf.ecore.EObject>(objects.size());
		reverse.addAll(objects);
		java.util.Collections.reverse(reverse);
		addObjectsToInterprete(reverse);
	}
	
	/**
	 * Adds the given object and all its children to the interpretation stack such
	 * that they are interpret in top down order.
	 */
	public void addObjectTreeToInterpreteTopDown(org.eclipse.emf.ecore.EObject root) {
		java.util.List<org.eclipse.emf.ecore.EObject> objects = new java.util.ArrayList<org.eclipse.emf.ecore.EObject>();
		objects.add(root);
		java.util.Iterator<org.eclipse.emf.ecore.EObject> it = root.eAllContents();
		while (it.hasNext()) {
			org.eclipse.emf.ecore.EObject eObject = (org.eclipse.emf.ecore.EObject) it.next();
			objects.add(eObject);
		}
		addObjectsToInterpreteInReverseOrder(objects);
	}
	
	public void addListener(org.emftext.language.java.resource.java.IJavaInterpreterListener newListener) {
		listeners.add(newListener);
	}
	
	public boolean removeListener(org.emftext.language.java.resource.java.IJavaInterpreterListener listener) {
		return listeners.remove(listener);
	}
	
	public org.eclipse.emf.ecore.EObject getNextObjectToInterprete() {
		return nextObjectToInterprete;
	}
	
	public java.util.Stack<org.eclipse.emf.ecore.EObject> getInterpretationStack() {
		return interpretationStack;
	}
	
	public void terminate() {
		interpretationStack.clear();
	}
	
	public Object getCurrentContext() {
		return currentContext;
	}
	
}
